package example.energy.coolemall;

import dcworms.schedframe.scheduling.ExecTask;
import schedframe.resources.ResourceStatus;
import schedframe.resources.computing.Core;
import schedframe.resources.computing.Processor;
import schedframe.resources.computing.profiles.energy.EnergyEvent;
import schedframe.resources.computing.profiles.energy.power.StandardPowerStateName;
import schedframe.resources.devices.PhysicalResource;
import schedframe.scheduling.manager.tasks.JobRegistry;
import schedframe.scheduling.manager.tasks.JobRegistryImpl;
import schedframe.scheduling.tasks.phases.PhaseBehaviour;
import example.energy.BaseEnergyEstimationPlugin;

public class ProcessorEnergyEstimationPlugin extends BaseEnergyEstimationPlugin  {

	public double estimatePowerConsumption(EnergyEvent event, JobRegistry jobRegistry,
			PhysicalResource resource) {
		
		/*double powerConsumption = 0;
		Processor cpu = (Processor)resource;
		if(resource.getPowerInterface().getPowerState().equals(StandardPowerStateName.OFF))
			powerConsumption = 0;
		else {
			if(resource.getStatus() == ResourceStatus.BUSY){
				try {
					powerConsumption = cpu.getPowerInterface().getPowerConsumption(cpu.getPowerInterface().getPState());
				} catch (NoSuchFieldException e) {
					Parameter param = cpu.getResourceCharacteristic().getParameters().get("maxPower");
					if(param != null)
						powerConsumption = Double.valueOf(param.get(0).getContent());
				}
			}
			else {
				try {
					powerConsumption = cpu.getPowerInterface().getPowerConsumption(StandardPowerStateName.ON);
				} catch (NoSuchFieldException e) {
					Parameter param = cpu.getResourceCharacteristic().getParameters().get("maxPower");
					if(param != null)
						powerConsumption = 0.7 * Double.valueOf(param.get(0).getContent());
				}
			}
		}
		return powerConsumption;*/
		double powerConsumption = 0;
		Processor proc = (Processor)resource;
		if(resource.getPowerInterface().getPowerState().equals(StandardPowerStateName.OFF))
			return 0;
		else {
			double processorLoad;
			double sumCoresLoad = 0;
			
			int nrOfThreadsOnCpu = 1;
			
			if(proc.getResourceCharacteristic().getParameters().get("product") != null){
				String productName = proc.getResourceCharacteristic().getParameters().get("product").get(0).getContent();
				if(productName.equals("Fusion G - T40N"))
					nrOfThreadsOnCpu = CoolEmAllTestbedMeasurements.Fusion_G_T40N_NR_OF_THREADS;
				else if(productName.equals("Atom - D510"))
					nrOfThreadsOnCpu = CoolEmAllTestbedMeasurements.Atom_D510_NR_OF_THREADS;
				else if(productName.equals("Atom - N2600"))
					nrOfThreadsOnCpu = CoolEmAllTestbedMeasurements.Atom_N2600_NR_OF_THREADS;
				else if(productName.equals("Core i7 - 2715QE"))
					nrOfThreadsOnCpu = CoolEmAllTestbedMeasurements.Core_i7_2715QE_NR_OF_THREADS;
				else if(productName.equals("Core i7 - 3615QE"))
					nrOfThreadsOnCpu = CoolEmAllTestbedMeasurements.Core_i7_3615QE_NR_OF_THREADS;
			}

			for(Core core: proc.getCores()){
				
				if(core.getStatus().equals(ResourceStatus.BUSY)){
					JobRegistry jr = new JobRegistryImpl(core.getFullName());
					for(ExecTask task: jr.getRunningTasks()){
						double cpuUsage = 100;
						double nrOfThreadsOfApplciation = 1;
						for(PhaseBehaviour pb: task.getCurrentResourceConsumption().getBehaviourList()){
							if(pb.getResouceName().equals("PM_CPU_Usage")){
								cpuUsage = pb.getUtilization();
							}
							if(pb.getResouceName().equals("PM_Threads")){
								nrOfThreadsOfApplciation = pb.getUtilization();
								break;
							}
						}
						sumCoresLoad = sumCoresLoad + cpuUsage * (nrOfThreadsOnCpu / nrOfThreadsOfApplciation);
					}
				}
			}
			processorLoad = 100 * sumCoresLoad/proc.getCores().size();
			double lowestLoadLevel = 100;
			double highestLoadLevel = 0;
			if(proc.getPowerInterface().getPState().getLoadPowerUsage().containsKey(new Double(processorLoad))){
				powerConsumption = proc.getPowerInterface().getPState().getLoadPowerUsage().get(new Double(processorLoad));
				//System.out.println("power from profile: " + powerConsumption);
			} else {
				for(Double load: proc.getPowerInterface().getPState().getLoadPowerUsage().keySet()){
					if(lowestLoadLevel > load){
						lowestLoadLevel = load;
					}
					if(highestLoadLevel < load){
						highestLoadLevel = load;
					}
				}
				if(processorLoad == 0){
					try{
						powerConsumption = proc.getPowerInterface().getSupportedPowerStates().get(0).getPowerUsage();
						//System.out.println("load = 0; power = idle power usage: " + powerConsumption);
					} catch (Exception e){
						powerConsumption = 0.7 * proc.getPowerInterface().getPState().getLoadPowerUsage().get(new Double(highestLoadLevel));
						//System.out.println("error load = 0; power = 0.7 max power: " + powerConsumption);
					}
				} else {
					
					double lowerLoadLevel = lowestLoadLevel;
					double higherLoadLevel = highestLoadLevel;
					
					try{
					
						for(Double load: proc.getPowerInterface().getPState().getLoadPowerUsage().keySet()){
							if(processorLoad > load){
								lowerLoadLevel = load;
							}
							else if(processorLoad < load){
								higherLoadLevel = load;
								break;
							}
						}
						double powerBelow;
						double powerAbove;
						double a;
						double b;
						if(lowerLoadLevel != higherLoadLevel)
						{
							powerBelow = proc.getPowerInterface().getPState().getLoadPowerUsage().get(lowerLoadLevel);
							powerAbove = proc.getPowerInterface().getPState().getLoadPowerUsage().get(higherLoadLevel);
							a = (powerAbove - powerBelow)/(higherLoadLevel - lowerLoadLevel);
							b = powerAbove - a * higherLoadLevel;
						} else {
							powerBelow = proc.getPowerInterface().getPState().getLoadPowerUsage().get(lowestLoadLevel);
							powerAbove = proc.getPowerInterface().getPState().getLoadPowerUsage().get(highestLoadLevel);
							a = (powerAbove - powerBelow)/(highestLoadLevel - lowestLoadLevel);
							b = powerAbove - a * highestLoadLevel;
						}
						powerConsumption = a * processorLoad + b;
						/*System.out.println("a" +lowestLoadLevel + ";" +highestLoadLevel);
						System.out.println("b" +lowerLoadLevel + ";" +higherLoadLevel);
						System.out.println("c" +powerBelow + ";" +powerAbove);
						System.out.println(resource.getFullName() + "load: " + processorLoad + "linear power estimation " + powerConsumption);*/
	
					} catch (Exception e){
						powerConsumption = 0.7 * proc.getPowerInterface().getPState().getLoadPowerUsage().get(new Double(highestLoadLevel));	
						//System.out.println("error; power = 0.7 max power: " + powerConsumption);
					}
				}
			}
		}
			

		powerConsumption = powerConsumption - CoolEmAllTestbedMeasurements.SINGLE_FAN_ON_POWER_CONSUMPTION;
		//System.out.println("estiamted power consumption: " + powerConsumption);
		return powerConsumption;
	}

}
