package schedframe.resources.computing;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import schedframe.resources.devices.Device;
import schedframe.resources.units.ResourceUnit;
import schedframe.resources.units.ResourceUnitName;

public class ComputingResourceCharacteristics extends ResourceCharacteristics{

	
	protected Map<ResourceUnitName, List<ResourceUnit>> resUnits;
	protected List<Device> devices;
	
	protected ComputingResourceCharacteristics(Builder<?> builder) {
		super(builder);
		this.resUnits = builder.resUnits;
		this.devices = builder.devices;
		// TODO Auto-generated constructor stub
	}

	public Map<ResourceUnitName, List<ResourceUnit>> getResourceUnits() {
		if(resUnits == null)
			return new HashMap<ResourceUnitName, List<ResourceUnit>>();
		return resUnits;
	}
	
	public ResourceUnit getResourceUnit(ResourceUnitName unitName) throws NoSuchFieldException{
		if(getResourceUnits().containsKey(unitName))
			return getResourceUnits().get(unitName).get(0);
		else throw new NoSuchFieldException("Resource unit " + unitName + 
				" is not available in this resource ");
	}
	
	public void addResourceUnit(ResourceUnit unit){ 
		if(resUnits == null){
			resUnits = new HashMap<ResourceUnitName, List<ResourceUnit>>(2);
		}
		List<ResourceUnit> list = null;
		if(resUnits.containsKey(unit.getName())){
			list = resUnits.get(unit.getName());
		} else {
			list = new ArrayList<ResourceUnit>(1);
			resUnits.put(unit.getName(), list);
		}
		list.add(unit);
	}
	

	
	public List<Device> getDevices(){
		if(devices == null)
			return new ArrayList<Device>();
		return devices;
	}
	
	public static abstract class Builder<T extends Builder<T>> extends ResourceCharacteristics.Builder<T> {

		protected Map<ResourceUnitName, List<ResourceUnit>> resUnits;
		protected List<Device> devices;
		
        public T resourceUnits(Map<ResourceUnitName, List<ResourceUnit>> units){this.resUnits = units; return self(); }
        public T resourceUnits(){this.resUnits = new HashMap<ResourceUnitName, List<ResourceUnit>>(2); return self(); }
        public T device(List<Device> dev){this.devices = dev; return self(); }

        public ComputingResourceCharacteristics build() {
            return new ComputingResourceCharacteristics(this);
        }
    }

    public static class Builder2 extends Builder<Builder2> {
        @Override
        protected Builder2 self() {
            return this;
        }
    }

    public static Builder<?> builder() {
        return new Builder2();
    }

}
